 //+------------------------------------------------------------------+
//|   #LR_Trend.mq4
//|   Continuously adjusting linear regression line
//+------------------------------------------------------------------+
#property  indicator_chart_window
#property  indicator_buffers 5
#property  indicator_color1  EMPTY
#property  indicator_color2  EMPTY
#property  indicator_color3  EMPTY
#property  indicator_color4  EMPTY
#property  indicator_color5  EMPTY

extern int   BarsBack = 13;
extern color SDUpColor = Blue;
extern color LRColor = Green;
extern color SDDownColor = Red;
extern int   Shift = 0;
extern bool  Update = true;
extern bool  ExtendLines = false;

int BoxSize; 
string BoxTrend;

string IndicatorName = "";
double StdDev;
double Diff, Diff2;

int StartIndex = 0, EndIndex = 0, ChartPeriod, Count;
int N, X, SumAllX, SumAllX2;
double B, M, Y, SumAllY, SumAllXY;
double AvgLR, AvgDiff, SumAllDiff2;
datetime StartTime, EndTime;
string strComment;

double SD2UpBuffer[];
double SD1UpBuffer[];
double LRBuffer[];
double SD1DnBuffer[];
double SD2DnBuffer[];
double TrendBuffer[];

double MyPoint;

int init()
{
    //**--Assign a 4 or 2 digit point value to MyPoint.
   //**--this stops problems with 5 Digit pricing --**//
   MyPoint = 0.0001;
   if (Digits < 4) MyPoint = 0.01;
  
   IndicatorBuffers(6);

   SetIndexStyle(0, DRAW_NONE);
   SetIndexBuffer(0, SD2UpBuffer);

   SetIndexStyle(1, DRAW_NONE);
   SetIndexBuffer(1, SD1UpBuffer);

   SetIndexStyle(2, DRAW_NONE);
   SetIndexBuffer(2, LRBuffer);

   SetIndexStyle(3, DRAW_NONE);
   SetIndexBuffer(3, SD1DnBuffer);

   SetIndexStyle(4, DRAW_NONE);
   SetIndexBuffer(4, SD2DnBuffer);
   
   SetIndexStyle(5, DRAW_NONE);
   SetIndexBuffer(5, TrendBuffer);

   ChartPeriod = Period();

   return(0);
}

int deinit()
{
   Comment("");

   ObjectsDeleteAll(0,OBJ_TREND);

   return(0);

}

//+------------------------------------------------------------------+
//| Calc linear regression channel
//+------------------------------------------------------------------+

int CalcLRC()
{
   int i,j,k,l;

   IndicatorName = "LR "+BarsBack+" "+Shift;
   if (Update == false && ObjectFind(IndicatorName) >= 0)
   {return;}

   StartIndex = BarsBack+Shift;

   EndIndex = Shift+1;

   X = 0;
   SumAllX = 0;
   SumAllY = 0;
   SumAllX2 = 0;
   SumAllXY = 0;
   N = StartIndex-EndIndex+1;

   for (i=StartIndex;i>=EndIndex;i--)
   {
      X++;
      SumAllX += X;
      SumAllY += Close[i];
      SumAllX2 += (X * X);
      SumAllXY += (X * Close[i]);
   }

   M = ((N * SumAllXY) - (SumAllX * SumAllY)) / ((N * SumAllX2) - (SumAllX * SumAllX));
   B = (SumAllY - (M * SumAllX)) / N;

//+--------------------------------------------------------
//| Standard deviation
//+------------------------------------------------------------------+
   AvgLR = 0;
   j = 0;
   for (i=StartIndex;i>=EndIndex;i--)
   {
      j++;
      LRBuffer[i] = (B + (M * j));     
      AvgLR += LRBuffer[i];      
   }   
   AvgLR /= N;
   
   StdDev = 0;
   AvgDiff = 0;
   SumAllDiff2 = 0;
   
   for (i=StartIndex;i>=EndIndex;i--)
   {
      SumAllDiff2 += ((Close[i] - AvgLR) * (Close[i] - AvgLR));
   }

   StdDev = MathSqrt(SumAllDiff2 / (N - 1));
   
//+------------------------------------------------------------------+
//| Load index buffers
//+------------------------------------------------------------------+
   
   double TrendValue;

   j = 0;
   for (i=StartIndex;i>=0;i--)
   {
      j++;
      TrendValue = (M/MyPoint);
      TrendBuffer[i] = TrendValue;
      LRBuffer[i] = (B + (M * j));     
      SD2UpBuffer[i] = LRBuffer[i] + (StdDev*2);
      SD2DnBuffer[i] = LRBuffer[i] - (StdDev*2);
      SD1UpBuffer[i] = LRBuffer[i] + StdDev;
      SD1DnBuffer[i] = LRBuffer[i] - StdDev;
   }
}

//+------------------------------------------------------------------+
//| Draw LR channel
//+------------------------------------------------------------------+

int DrawLRC()
{
//   IndicatorName = " "+BarsBack+" "+Shift+TimeToStr(StartTime,TIME_DATE|TIME_MINUTES);
   IndicatorName = " "+BarsBack+" "+Shift;

   ObjectDelete("SD2Up"+IndicatorName);
   ObjectDelete("SD1Up"+IndicatorName);
   ObjectDelete("LR"+IndicatorName);
   ObjectDelete("SD1Dn"+IndicatorName);
   ObjectDelete("SD2Dn"+IndicatorName);
   ObjectDelete("VL1"+IndicatorName);
   ObjectDelete("VL2"+IndicatorName);
   
   StartTime = Time[StartIndex];
   EndTime = Time[EndIndex];

   ObjectCreate("SD2Up"+IndicatorName,OBJ_TREND,0,StartTime,SD2UpBuffer[StartIndex],EndTime+(ChartPeriod*120),SD2UpBuffer[EndIndex]+(M*2));
   ObjectSet("SD2Up"+IndicatorName,OBJPROP_COLOR,SDUpColor); 
   ObjectSet("SD2Up"+IndicatorName,OBJPROP_WIDTH,1); 
   ObjectSet("SD2Up"+IndicatorName,OBJPROP_STYLE,STYLE_SOLID); 
   ObjectSet("SD2Up"+IndicatorName,OBJPROP_RAY,ExtendLines);

   ObjectCreate("SD1Up"+IndicatorName,OBJ_TREND,0,StartTime,SD1UpBuffer[StartIndex],EndTime+(ChartPeriod*120),SD1UpBuffer[EndIndex]+(M*2));
   ObjectSet("SD1Up"+IndicatorName,OBJPROP_COLOR,SDUpColor); 
   ObjectSet("SD1Up"+IndicatorName,OBJPROP_WIDTH,1); 
   ObjectSet("SD1Up"+IndicatorName,OBJPROP_STYLE,STYLE_DOT); 
   ObjectSet("SD1Up"+IndicatorName,OBJPROP_RAY,ExtendLines);

   ObjectCreate("LR"+IndicatorName,OBJ_TREND,0,StartTime,LRBuffer[StartIndex],EndTime+(ChartPeriod*120),LRBuffer[EndIndex]+(M*2));
   ObjectSet("LR"+IndicatorName,OBJPROP_COLOR,LRColor); 
   ObjectSet("LR"+IndicatorName,OBJPROP_WIDTH,1); 
   ObjectSet("LR"+IndicatorName,OBJPROP_STYLE,STYLE_SOLID); 
   ObjectSet("LR"+IndicatorName,OBJPROP_RAY,ExtendLines);

   ObjectCreate("SD1Dn"+IndicatorName,OBJ_TREND,0,StartTime,SD1DnBuffer[StartIndex],EndTime+(ChartPeriod*120),SD1DnBuffer[EndIndex]+(M*2));
   ObjectSet("SD1Dn"+IndicatorName,OBJPROP_COLOR,SDDownColor); 
   ObjectSet("SD1Dn"+IndicatorName,OBJPROP_WIDTH,1); 
   ObjectSet("SD1Dn"+IndicatorName,OBJPROP_STYLE,STYLE_DOT); 
   ObjectSet("SD1Dn"+IndicatorName,OBJPROP_RAY,ExtendLines);

   ObjectCreate("SD2Dn"+IndicatorName,OBJ_TREND,0,StartTime,SD2DnBuffer[StartIndex],EndTime+(ChartPeriod*120),SD2DnBuffer[EndIndex]+(M*2));
   ObjectSet("SD2Dn"+IndicatorName,OBJPROP_COLOR,SDDownColor); 
   ObjectSet("SD2Dn"+IndicatorName,OBJPROP_WIDTH,1); 
   ObjectSet("SD2Dn"+IndicatorName,OBJPROP_STYLE,STYLE_SOLID); 
   ObjectSet("SD2Dn"+IndicatorName,OBJPROP_RAY,ExtendLines);

   ObjectCreate("VL1"+IndicatorName,OBJ_TREND,0,StartTime,SD2UpBuffer[StartIndex],StartTime,SD2DnBuffer[StartIndex]);
   ObjectSet("VL1"+IndicatorName,OBJPROP_COLOR,LRColor); 
   ObjectSet("VL1"+IndicatorName,OBJPROP_WIDTH,1); 
   ObjectSet("VL1"+IndicatorName,OBJPROP_STYLE,STYLE_DOT); 
   ObjectSet("VL1"+IndicatorName,OBJPROP_RAY,false);

   ObjectCreate("VL2"+IndicatorName,OBJ_TREND,0,EndTime+(ChartPeriod*120),SD2UpBuffer[EndIndex]+(M*2),EndTime+(ChartPeriod*120),SD2DnBuffer[EndIndex]+(M*2));
   ObjectSet("VL2"+IndicatorName,OBJPROP_COLOR,LRColor); 
   ObjectSet("VL2"+IndicatorName,OBJPROP_WIDTH,1); 
   ObjectSet("VL2"+IndicatorName,OBJPROP_STYLE,STYLE_DOT); 
   ObjectSet("VL2"+IndicatorName,OBJPROP_RAY,false);
}

//+------------------------------------------------------------------+
//| Indicator start function
//+------------------------------------------------------------------+

int start()
{
   CalcLRC();
   DrawLRC();

   BoxTrend = DoubleToStr(M/MyPoint,4);
   BoxSize = MathRound((SD2UpBuffer[0]-SD2DnBuffer[0])/MyPoint); 

   
   return(0);
}
//+------------------------------------------------------------------+

